"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var headers_1 = require("../helpers/headers");
var error_1 = require("../helpers/error");
var url_1 = require("../helpers/url");
var cookie_1 = require("../helpers/cookie");
var util_1 = require("../helpers/util");
//  整个流程分为 7 步：
//  创建一个 request 实例。
//  执行 request.open 方法初始化。
//  执行 configureRequest 配置 request 对象。
//  执行 addEvents 给 request 添加事件处理函数。
//  执行 processHeaders 处理请求 headers。
//  执行 processCancel 处理请求取消逻辑。
//  执行 request.send 方法发送请求。
function xhr(config) {
    // Promise封装一个异步请求，用户可以通过then处理服务端返回的数据
    return new Promise(function (resolve, reject) {
        // config是用户设置的请求信息
        var _a = config.data, data = _a === void 0 ? null : _a, url = config.url, _b = config.method, method = _b === void 0 ? 'get' : _b, headers = config.headers, responseType = config.responseType, timeout = config.timeout, cancelToken = config.cancelToken, withCredentials = config.withCredentials, xsrfCookieName = config.xsrfCookieName, xsrfHeaderName = config.xsrfHeaderName, onDownloadProgress = config.onDownloadProgress, onUploadProgress = config.onUploadProgress, auth = config.auth, validateStatus = config.validateStatus;
        var request = new XMLHttpRequest();
        // true开启异步
        request.open(method.toUpperCase(), url, true);
        // 配置requestConfig对象
        configRequest();
        // 给request添加事件处理函数
        addEvents();
        // 处理headers
        processHeaders();
        // 处理请求取消逻辑
        processCancel();
        request.send(data);
        function configRequest() {
            // 如果用户指定了返回数据的类型
            if (responseType) {
                request.responseType = responseType;
            }
            // 如果用户指定请求的超时时间，超过一定时间没有响应触发timeout事件
            if (timeout) {
                request.timeout = timeout;
            }
            // 如果用户设置withCredentials，发送请求时会携带cookie
            if (withCredentials) {
                request.withCredentials = withCredentials;
            }
        }
        function addEvents() {
            request.onreadystatechange = function handleLoad() {
                if (request.readyState !== 4) {
                    return;
                }
                if (request.status === 0) {
                    return;
                }
                // 获取所有响应头
                var responseHeaders = (0, headers_1.parseHeaders)(request.getAllResponseHeaders());
                // 获取返回的响应数据
                var responseData = responseType && responseType !== 'text' ? request.response : request.responseText;
                var response = {
                    data: responseData,
                    status: request.status,
                    statusText: request.statusText,
                    headers: responseHeaders,
                    config: config,
                    request: request
                };
                handleResponse(response);
            };
            // 处理网络错误
            request.onerror = function handleError() {
                reject((0, error_1.createError)('Network Error', config, null, request));
            };
            // 处理超时
            request.ontimeout = function handleTimeout() {
                reject((0, error_1.createError)("Timeout of ".concat(config.timeout, " ms exceeded"), config, 'ECONNABORTED', request));
            };
            // 下载和上传的进度
            if (onDownloadProgress) {
                request.onprogress = onDownloadProgress;
            }
            if (onUploadProgress) {
                request.upload.onprogress = onUploadProgress;
            }
        }
        function processHeaders() {
            // 如果请求是个 FormData 类型，则删除 headers['Content-Type'] 让浏览器自动为请求带上合适的 Content-Type
            if ((0, util_1.isFormData)(data)) {
                delete headers['Content-Type'];
            }
            // 读取和设置cookie
            if ((withCredentials || (0, url_1.isURLSameOrigin)(url)) && xsrfCookieName) {
                var xsrfValue = cookie_1.default.read(xsrfCookieName);
                if (xsrfValue) {
                    headers[xsrfHeaderName] = xsrfValue;
                }
            }
            // 如果用户设置了auth,如果配置了就把该属性经过base64 加密后添加到请求 headers 中的 Authorization 属性上
            if (auth) {
                headers['Authorization'] = "Basic ".concat(btoa("".concat(auth.username, " : ").concat(auth.password)));
            }
            // 如果传入的data为空，请求headers配置Content-Type是没有意义的，所以删除掉
            Object.keys(headers).forEach(function (name) {
                if (data === null && name.toLocaleLowerCase() === 'content-type') {
                    delete headers[name];
                }
                else {
                    request.setRequestHeader(name, headers[name]);
                }
            });
        }
        function processCancel() {
            // 首先判断用户是否配置的cancelToken，如果没有配置，表示没有取消请求这项需求；
            // 如果配置cancelToken，并且当外部调用了请求取消触发函数，此时cancelToken.promise会变成resolved 状态，然后就会执行then函数，在then函数内部调用XMLHttpRequest对象上的abort()取消请求
            if (cancelToken) {
                cancelToken.promise
                    .then(function (reason) {
                    request.abort();
                    reject(reason);
                })
                    .catch(function () {
                    // do nothing
                });
            }
        }
        function handleResponse(response) {
            // 自定义合法状态码
            if (!validateStatus || validateStatus(response.status)) {
                resolve(response);
            }
            else {
                reject((0, error_1.createError)("Request failed with status code ".concat(response.status), config, null, request, response));
            }
        }
    });
}
exports.default = xhr;
//# sourceMappingURL=xhr.js.map